@@ -26,8 +26,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -54,8 +57,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -81,11 +93,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_winconfig.h,v 1.14 2004/03/12 01:59:04 fyodor Exp $ */
+/* $Id: nbase_winconfig.h 2966 2005-12-05 21:31:10Z fyodor $ */
 
 #ifndef NBASE_WINCONFIG_H
 #define NBASE_WINCONFIG_H
@@ -93,7 +106,8 @@
 /* It doesn't really have strucct IP, but we use a different one instead
 	of the one that comes with Nmap */
 #define HAVE_STRUCT_IP 1
-#define HAVE_STRUCT_ICMP 1
+/* #define HAVE_STRUCT_ICMP 1 */
+#define HAVE_STRNCASECMP 1
 #define HAVE_IP_IP_SUM 1
 #define STDC_HEADERS 1
 #define HAVE_STRING_H 1
@@ -105,12 +119,16 @@
 #define HAVE_STRERROR 1
 #define HAVE_SYS_SOCKIO_H 1
 #define HAVE_ERRNO_H 1
+#define HAVE_GAI_STRERROR 1
 /* #define HAVE_STRCASESTR 1 */
 #define HAVE_STRCASECMP 1
 #define HAVE_NETINET_IN_SYSTEM_H 1
 #define HAVE_NETINET_IF_ETHER_H 1
 #define HAVE_SYS_STAT_H 1
 
+#define HAVE_GETADDRINFO 1
+#define HAVE_GETNAMEINFO 1
+
 #define HAVE_SNPRINTF 1
 #define HAVE_VASPRINTF 1
 #define HAVE_VSNPRINTF 1
@@ -123,6 +141,10 @@
 #define HAVE_AF_INET6 1
 #define HAVE_SOCKADDR_STORAGE 1
 
+#ifdef __GNUC__
+#define bzero(addr, num) __builtin_memset (addr, '\0', num)
+#else
 #define __attribute__(x)
+#endif
 
 #endif /* NBASE_WINCONFIG_H */
