@@ -26,8 +26,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -54,8 +57,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -81,11 +93,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: osscan.cc,v 1.21 2004/04/19 02:02:23 fyodor Exp $ */
+/* $Id: osscan.cc 2891 2005-10-01 23:50:27Z fyodor $ */
 
 
 #include "osscan.h"
@@ -104,10 +117,6 @@
 #endif
 
 extern NmapOps o;
-/*  predefined filters -- I need to kill these globals at some pont. */
-extern unsigned long flt_dsthost, flt_srchost;
-extern unsigned short flt_baseport;
-
 
 FingerPrint *get_fingerprint(Target *target, struct seq_info *si) {
 FingerPrint *FP = NULL, *FPtmp = NULL;
@@ -131,7 +140,7 @@ int testno;
 int  timeout;
 int avnum;
 unsigned int sequence_base;
-unsigned int openport;
+unsigned long openport;
 unsigned int bytes;
 unsigned int closedport = 31337;
 Port *tport = NULL;
@@ -149,8 +158,10 @@ int seq_packets_sent = 0;
 int seq_response_num; /* response # for sequencing */
 double avg_ts_hz = 0.0; /* Avg. amount that timestamps incr. each second */
 struct link_header linkhdr;
+struct eth_nfo eth;
+struct eth_nfo *ethptr; // for passing to send_ functions 
 
-if (target->timedout)
+if (target->timedOut(NULL))
   return NULL;
 
 /* The seqs must start out as zero for the si struct */
@@ -162,11 +173,24 @@ si->lastboot = 0;
 /* Init our fingerprint tests to each be NULL */
 memset(FPtests, 0, sizeof(FPtests)); 
 get_random_bytes(&sequence_base, sizeof(unsigned int));
-/* Init our raw socket */
- if ((rawsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 )
-   pfatal("socket trobles in get_fingerprint");
- unblock_socket(rawsd);
- broadcast_socket(rawsd);
+ if ((o.sendpref & PACKET_SEND_ETH) &&  target->ifType() == devt_ethernet) {
+   memcpy(eth.srcmac, target->SrcMACAddress(), 6);
+   memcpy(eth.dstmac, target->NextHopMACAddress(), 6);
+   eth.ethsd = eth_open(target->deviceName());
+   rawsd = -1;
+   ethptr = &eth;
+  } else {
+    /* Init our raw socket */
+    if ((rawsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 )
+      pfatal("socket troubles in get_fingerprint");
+    unblock_socket(rawsd);
+    broadcast_socket(rawsd);
+#ifndef WIN32
+    sethdrinclude(rawsd);
+#endif
+    ethptr = NULL;
+    eth.ethsd = NULL;
+  }
 
  /* Now for the pcap opening nonsense ... */
  /* Note that the snaplen is 152 = 64 byte max IPhdr + 24 byte max link_layer
@@ -176,17 +200,14 @@ get_random_bytes(&sequence_base, sizeof(unsigned int));
 ossofttimeout = MAX(200000, target->to.timeout);
 oshardtimeout = MAX(500000, 5 * target->to.timeout);
 
- pd = my_pcap_open_live(target->device, /*650*/ 8192,  (o.spoofsource)? 1 : 0, (ossofttimeout + 500)/ 1000);
+ pd = my_pcap_open_live(target->deviceName(), /*650*/ 8192,  (o.spoofsource)? 1 : 0, (ossofttimeout + 500)/ 1000);
 
-if (o.debugging)
+if (o.debugging > 1)
    log_write(LOG_STDOUT, "Wait time is %dms\n", (ossofttimeout +500)/1000);
 
- flt_srchost = target->v4host().s_addr;
- flt_dsthost = target->v4source().s_addr;
-
 snprintf(filter, sizeof(filter), "dst host %s and (icmp or (tcp and src host %s))", inet_ntoa(target->v4source()), target->targetipstr());
  
- set_pcap_filter(target, pd, flt_icmptcp, filter);
+ set_pcap_filter(target->deviceName(), pd, filter);
  target->osscan_performed = 1; /* Let Nmap know that we did try an OS scan */
 
  /* Lets find an open port to use */
@@ -203,7 +224,7 @@ snprintf(filter, sizeof(filter), "dst host %s and (icmp or (tcp and src host %s)
  if ((tport = target->ports.nextPort(NULL, IPPROTO_TCP, PORT_CLOSED, false))) {
    closedport = tport->portno;
    target->FPR->osscan_closedtcpport = tport->portno;
- } else if ((tport = target->ports.nextPort(NULL, IPPROTO_TCP, PORT_UNFIREWALLED, false))) {
+ } else if ((tport = target->ports.nextPort(NULL, IPPROTO_TCP, PORT_UNFILTERED, false))) {
    /* Well, we will settle for unfiltered */
    closedport = tport->portno;
  } else {
@@ -211,7 +232,7 @@ snprintf(filter, sizeof(filter), "dst host %s and (icmp or (tcp and src host %s)
  }
 
 if (o.verbose && openport != (unsigned long) -1)
-  log_write(LOG_STDOUT, "For OSScan assuming that port %d is open and port %d is closed and neither are firewalled\n", openport, closedport);
+  log_write(LOG_STDOUT, "For OSScan assuming port %lu is open, %d is closed, and neither are firewalled\n", openport, closedport);
 
  current_port = o.magic_port + NUM_SEQ_SAMPLES +1;
  
@@ -225,28 +246,31 @@ if (o.verbose && openport != (unsigned long) -1)
      /* Test 1 */
      if (!FPtests[1]) {     
        if (o.scan_delay) enforce_scan_delay(NULL);
-       send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, current_port, 
-			   openport, sequence_base, 0,TH_ECE|TH_SYN, 0, (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
+       send_tcp_raw_decoys(rawsd, ethptr, target->v4hostip(), o.ttl, 
+			   current_port, openport, sequence_base, 0, 
+			   TH_ECE|TH_SYN, 0, (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
      }
      
      /* Test 2 */
      if (!FPtests[2]) {     
        if (o.scan_delay) enforce_scan_delay(NULL);
-       send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, current_port +1, 
+       send_tcp_raw_decoys(rawsd, ethptr, target->v4hostip(), o.ttl, 
+			   current_port +1, 
 			   openport, sequence_base, 0,0, 0, (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
      }
 
      /* Test 3 */
      if (!FPtests[3]) {     
        if (o.scan_delay) enforce_scan_delay(NULL);
-       send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, current_port +2, 
+       send_tcp_raw_decoys(rawsd, ethptr, target->v4hostip(), o.ttl, current_port +2, 
 			   openport, sequence_base, 0,TH_SYN|TH_FIN|TH_URG|TH_PUSH, 0,(u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
      }
 
      /* Test 4 */
      if (!FPtests[4]) {     
        if (o.scan_delay) enforce_scan_delay(NULL);
-       send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, current_port +3, 
+       send_tcp_raw_decoys(rawsd, ethptr, target->v4hostip(), o.ttl, 
+			   current_port +3, 
 			   openport, sequence_base, 0,TH_ACK, 0, (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
      }
    }
@@ -254,48 +278,47 @@ if (o.verbose && openport != (unsigned long) -1)
    /* Test 5 */
    if (!FPtests[5]) {   
      if (o.scan_delay) enforce_scan_delay(NULL);
-     send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, current_port +4,
+     send_tcp_raw_decoys(rawsd, ethptr, target->v4hostip(), o.ttl, 
+			 current_port +4,
 			 closedport, sequence_base, 0,TH_SYN, 0, (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
    }
 
      /* Test 6 */
    if (!FPtests[6]) {   
      if (o.scan_delay) enforce_scan_delay(NULL);
-     send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, current_port +5, 
+     send_tcp_raw_decoys(rawsd, ethptr, target->v4hostip(), o.ttl, 
+			 current_port +5, 
 			 closedport, sequence_base, 0,TH_ACK, 0, (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
    }
 
      /* Test 7 */
    if (!FPtests[7]) {
      if (o.scan_delay) enforce_scan_delay(NULL);   
-     send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, current_port +6, 
+     send_tcp_raw_decoys(rawsd, ethptr, target->v4hostip(), o.ttl, 
+			 current_port +6, 
 			 closedport, sequence_base, 0,TH_FIN|TH_PUSH|TH_URG, 0, (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
    }
 
    /* Test 8 */
    if (!FPtests[8]) {
      if (o.scan_delay) enforce_scan_delay(NULL);
-     upi = send_closedudp_probe(rawsd, target->v4hostip(), o.magic_port, closedport);
+     upi = send_closedudp_probe(rawsd, ethptr, target->v4hostip(), o.magic_port, closedport);
    }
    gettimeofday(&t1, NULL);
    timeout = 0;
 
    /* Insure we haven't overrun our allotted time ... */
-   if (o.host_timeout && (TIMEVAL_MSEC_SUBTRACT(t1, target->host_timeout) >= 0))
-     {
-       target->timedout = 1;
-       goto osscan_timedout;
-     }
+   if (target->timedOut(&t1))
+     goto osscan_timedout;
+
    while(( ip = (struct ip*) readip_pcap(pd, &bytes, oshardtimeout, NULL, &linkhdr)) && !timeout) {
      gettimeofday(&t2, NULL);
      if (TIMEVAL_SUBTRACT(t2,t1) > oshardtimeout) {
        timeout = 1;
      }
-     if (o.host_timeout && (TIMEVAL_MSEC_SUBTRACT(t2, target->host_timeout) >= 0))
-       {
-	 target->timedout = 1;
-	 goto osscan_timedout;
-       }
+
+     if (target->timedOut(&t2))
+       goto osscan_timedout;
 
      if (bytes < (4 * ip->ip_hl) + 4U || bytes < 20)
        continue;
@@ -349,19 +372,25 @@ if (o.verbose && openport != (unsigned long) -1)
    seq_packets_sent = 0;
    while (seq_packets_sent < NUM_SEQ_SAMPLES) {
      if (o.scan_delay) enforce_scan_delay(NULL);
-     send_tcp_raw_decoys(rawsd, target->v4hostip(), o.ttl, 
+     if (seq_packets_sent > 0) {
+       gettimeofday(&t1, NULL);
+       int remaining_us = 110000 - TIMEVAL_SUBTRACT(t1, seq_send_times[seq_packets_sent - 1]);
+       if (remaining_us > 0) {
+	 /* Need to spend at least .5 seconds in sending all packets to
+	    reliably detect 2HZ timestamp sequencing */
+	 usleep(remaining_us);
+       }
+     }
+     send_tcp_raw_decoys(rawsd, ethptr, target->v4hostip(), o.ttl, 
 			 o.magic_port + seq_packets_sent + 1, 
 			 openport, 
 			 sequence_base + seq_packets_sent + 1, 0, 
 			 TH_SYN, 0 , (u8 *) "\003\003\012\001\002\004\001\011\010\012\077\077\077\077\000\000\000\000\000\000" , 20, NULL, 0);
-     if (!o.scan_delay)
-       usleep( MAX(110000, target->to.srtt)); /* Main reason we wait so long is that we need to spend more than .5 seconds to detect 2HZ timestamp sequencing -- this also should make ISN sequencing more regular */
-   
      gettimeofday(&seq_send_times[seq_packets_sent], NULL);
+     t1 = seq_send_times[seq_packets_sent];
      seq_packets_sent++;
-     
-     /* Now we collect  the replies */
-     
+   
+     /* Now we collect the replies */
      while(si->responses < seq_packets_sent && !timeout) {
        
        if (seq_packets_sent == NUM_SEQ_SAMPLES)
@@ -372,11 +401,9 @@ if (o.verbose && openport != (unsigned long) -1)
        /*     error("DEBUG: got a response (len=%d):\n", bytes);  */
        /*     lamont_hdump((unsigned char *) ip, bytes); */
        /* Insure we haven't overrun our allotted time ... */
-       if (o.host_timeout && (TIMEVAL_MSEC_SUBTRACT(t2, target->host_timeout) >= 0))
-	 {
-	   target->timedout = 1;
-	   goto osscan_timedout;
-	 }
+       if (target->timedOut(&t2))
+	 goto osscan_timedout;
+
        if (!ip) { 
 	 if (seq_packets_sent < NUM_SEQ_SAMPLES)
 	   break;
@@ -398,13 +425,15 @@ if (o.verbose && openport != (unsigned long) -1)
        if (ip->ip_p == IPPROTO_TCP) {
 	 /*       readtcppacket((char *) ip, ntohs(ip->ip_len));  */
 	 tcp = ((struct tcphdr *) (((char *) ip) + 4 * ip->ip_hl));
-	 if (ntohs(tcp->th_dport) < o.magic_port || ntohs(tcp->th_dport) - o.magic_port > NUM_SEQ_SAMPLES || ntohs(tcp->th_sport) != openport) {
+	 if (ntohs(tcp->th_dport) < o.magic_port || 
+	     ntohs(tcp->th_dport) - o.magic_port > NUM_SEQ_SAMPLES || 
+	     ntohs(tcp->th_sport) != openport) {
 	   continue;
 	 }
 	 if ((tcp->th_flags & TH_RST)) {
 	   /*	 readtcppacket((char *) ip, ntohs(ip->ip_len));*/	 
 	   if (si->responses == 0) {	 
-	     fprintf(stderr, "WARNING:  RST from port %hu -- is this port really open?\n", openport);
+	     fprintf(stderr, "WARNING:  RST from port %lu -- is this port really open?\n", openport);
 	     /* We used to quit in this case, but left-overs from a SYN
 		scan or lame-ass TCP wrappers can cause this! */
 	   } 
@@ -419,25 +448,28 @@ if (o.verbose && openport != (unsigned long) -1)
 	     /* BzzT! Value out of range */
 	     if (o.debugging) {
 	       error("Unable to associate os scan response with sent packet (received ack: %lX; sequence base: %lX. Packet:", (unsigned long) ntohl(tcp->th_ack), (unsigned long) sequence_base);
-	       readtcppacket((unsigned char *)ip,BSDUFIX(ip->ip_len));
+	       readtcppacket((unsigned char *)ip,ntohs(ip->ip_len));
 	     }
 	     seq_response_num = si->responses;
 	   }
-	   si->responses++;
-	   si->seqs[seq_response_num] = ntohl(tcp->th_seq); /* TCP ISN */
-	   si->ipids[seq_response_num] = ntohs(ip->ip_id);
-	   if ((gettcpopt_ts(tcp, &timestamp, NULL) == 0))
-	     si->ts_seqclass = TS_SEQ_UNSUPPORTED;
-	   else {
-	     if (timestamp == 0) {
-	       si->ts_seqclass = TS_SEQ_ZERO;
+	   if (si->seqs[seq_response_num] == 0) {
+	     /* New response found! */
+	     si->responses++;
+	     si->seqs[seq_response_num] = ntohl(tcp->th_seq); /* TCP ISN */
+	     si->ipids[seq_response_num] = ntohs(ip->ip_id);
+	     if ((gettcpopt_ts(tcp, &timestamp, NULL) == 0))
+	       si->ts_seqclass = TS_SEQ_UNSUPPORTED;
+	     else {
+	       if (timestamp == 0) {
+		 si->ts_seqclass = TS_SEQ_ZERO;
+	       }
+	     }
+	     si->timestamps[seq_response_num] = timestamp;
+	     /*           printf("Response #%d -- ipid=%hu ts=%i\n", seq_response_num, ntohs(ip->ip_id), timestamp); */
+	     if (si->responses > 1) {
+	       seq_diffs[si->responses-2] = MOD_DIFF(ntohl(tcp->th_seq), si->seqs[si->responses-2]);
 	     }
 	   }
-	   si->timestamps[seq_response_num] = timestamp;
-	   /*           printf("Response #%d -- ipid=%hu ts=%i\n", seq_response_num, ntohs(ip->ip_id), timestamp); */
-	   if (si->responses > 1) {
-	     seq_diffs[si->responses-2] = MOD_DIFF(ntohl(tcp->th_seq), si->seqs[si->responses-2]);
-	   }      
 	 }
        }
      }
@@ -715,9 +747,13 @@ for(i=0; i < 9; i++) {
  if (last) FPtests[last]->next = NULL;
  
  osscan_timedout:
- if (target->timedout)
+ if (target->timedOut(NULL))
    FP = NULL;
- close(rawsd);
+ if (rawsd >= 0)
+   close(rawsd);
+ if (ethptr) {
+   eth_close(ethptr->ethsd);
+ }
  pcap_close(pd);
  return FP;
 }
@@ -893,7 +929,7 @@ double compare_fingerprints(FingerPrint *referenceFP, FingerPrint *observedFP,
 }
 
 /* Takes a fingerprint and looks for matches inside reference_FPs[].
-   The results are stored in in FPR (which must point to an instantiated
+   The results are stored in FPR (which must point to an instantiated
    FingerPrintResults class) -- results will be reverse-sorted by
    accuracy.  No results below accuracy_threshhold will be included.
    The max matches returned is the maximum that fits in a
@@ -1129,17 +1165,30 @@ int i;
 struct timeval now;
 double bestacc;
 int bestaccidx;
+ int starttimems = 0;
 
- if (target->timedout)
+ if (target->timedOut(NULL))
    return 1;
  
+#ifdef WIN32
+  if (target->ifType() == devt_loopback) {
+    log_write(LOG_STDOUT, "Skipping OS Scan against %s because it doesn't work against your own machine (localhost)\n", target->NameIP());
+    return 1;
+  }
+#endif
+
+ if (o.debugging > 2) {
+   starttimems = o.TimeSinceStartMS();
+   log_write(LOG_STDOUT|LOG_NORMAL|LOG_SKID, "Initiating OS Detection against %s at %.3fs\n", target->targetipstr(), starttimems / 1000.0);
+ }
+
  if (target->FPR == NULL)
    target->FPR = new FingerPrintResults;
 
  memset(si, 0, sizeof(si));
  if (target->ports.state_counts_tcp[PORT_OPEN] == 0 ||
      (target->ports.state_counts_tcp[PORT_CLOSED] == 0 &&
-      target->ports.state_counts_tcp[PORT_UNFIREWALLED] == 0)) {
+      target->ports.state_counts_tcp[PORT_UNFILTERED] == 0)) {
    if (o.osscan_limit) {
      if (o.verbose)
        log_write(LOG_STDOUT|LOG_NORMAL|LOG_SKID, "Skipping OS Scan due to absence of open (or perhaps closed) ports\n");
@@ -1150,17 +1199,12 @@ int bestaccidx;
  }
 
  for(itry=0; itry < 3; itry++) {
-   if (o.host_timeout) {   
-     gettimeofday(&now, NULL);
-     if (target->timedout || TIMEVAL_MSEC_SUBTRACT(now, target->host_timeout) >= 0)
-       {
-	 target->timedout = 1;
-	 return 1;
-       }
-   }
-   target->FPR->FPs[itry] = get_fingerprint(target, &si[itry]); 
-   if (target->timedout)
+   gettimeofday(&now, NULL);
+   if (target->timedOut(&now))
      return 1;
+
+   target->FPR->FPs[itry] = get_fingerprint(target, &si[itry]); 
+
    match_fingerprint(target->FPR->FPs[itry], &FP_matches[itry], 
 		     o.reference_FPs, OSSCAN_GUESS_THRESHOLD);
    if (FP_matches[itry].overall_results == OSSCAN_SUCCESS && 
@@ -1210,6 +1254,9 @@ int bestaccidx;
    match_fingerprint(target->FPR->FPs[target->FPR->goodFP], target->FPR, 
 		     o.reference_FPs, OSSCAN_GUESS_THRESHOLD);
 
+ if (o.debugging > 2) {
+   log_write(LOG_STDOUT|LOG_NORMAL|LOG_SKID, "Completed OS Detection against %s at %.3fs (took %.3fs)\n", target->targetipstr(), o.TimeSinceStartMS() / 1000.0, (o.TimeSinceStartMS() - starttimems) / 1000.0);
+ }
  
  return 1;
 }
@@ -1218,19 +1265,26 @@ int bestaccidx;
    top of a fingerprint.  Gives info which might be useful when the
    FPrint is submitted (eg Nmap version, etc).  Result is written (up
    to ostrlen) to the ostr var passed in */
-void WriteSInfo(char *ostr, int ostrlen, int openport, int closedport) {
+void WriteSInfo(char *ostr, int ostrlen, int openport, int closedport, 
+		const u8 *mac) {
   struct tm *ltime;
   time_t timep;
-  
+  char macbuf[16];
   timep = time(NULL);
   ltime = localtime(&timep);
 
-  snprintf(ostr, ostrlen, "SInfo(V=%s%%P=%s%%D=%d/%d%%Time=%X%%O=%d%%C=%d)\n", 
+  macbuf[0] = '\0';
+  if (mac)
+    snprintf(macbuf, sizeof(macbuf), "%%M=%02X%02X%02X", mac[0], mac[1], 
+	     mac[2]);
+
+  snprintf(ostr, ostrlen, "SInfo(V=%s%%P=%s%%D=%d/%d%%Tm=%X%%O=%d%%C=%d%s)\n", 
 	   NMAP_VERSION, NMAP_PLATFORM, ltime->tm_mon + 1, ltime->tm_mday, 
-	   (int) timep, openport, closedport);
+	   (int) timep, openport, closedport, macbuf);
 }
 
-char *mergeFPs(FingerPrint *FPs[], int numFPs, int openport, int closedport) {
+char *mergeFPs(FingerPrint *FPs[], int numFPs, int openport, int closedport, 
+	       const u8 *mac) {
 static char str[10240];
 struct AVal *AV;
 FingerPrint *currentFPs[32];
@@ -1250,7 +1304,7 @@ for(i=0; i < numFPs; i++) {
 }
 
 /* Lets start by writing the fake "Info" test for submitting fingerprints */
- WriteSInfo(str, sizeof(str), openport, closedport); 
+ WriteSInfo(str, sizeof(str), openport, closedport, mac); 
  p = p + strlen(str);
 
 do {
@@ -1652,7 +1706,8 @@ return AVs;
 }
 
 
-struct udpprobeinfo *send_closedudp_probe(int sd, const struct in_addr *victim,
+struct udpprobeinfo *send_closedudp_probe(int sd, struct eth_nfo *eth,
+					  const struct in_addr *victim,
 					  u16 sport, u16 dport) {
 
 static struct udpprobeinfo upi;
@@ -1667,7 +1722,6 @@ int datalen = 300;
 unsigned char *data = packet + 28;
 unsigned short realcheck; /* the REAL checksum */
 int res;
-struct sockaddr_in sock;
 int decoy;
 struct pseudo_udp_hdr {
   struct in_addr source;
@@ -1683,96 +1737,78 @@ memset(data, patternbyte, datalen);
 while(!id) id = get_random_uint();
 
 /* check that required fields are there and not too silly */
-if ( !victim || !sport || !dport || sd < 0) {
+if ( !victim || !sport || !dport || (!eth && sd < 0)) {
   fprintf(stderr, "send_closedudp_probe: One or more of your parameters suck!\n");
   return NULL;
 }
 
 if (!myttl)  myttl = (time(NULL) % 14) + 51;
-/* It was a tough decision whether to do this here for every packet
-   or let the calling function deal with it.  In the end I grudgingly decided
-   to do it here and potentially waste a couple microseconds... */
-sethdrinclude(sd); 
 
- for(decoy=0; decoy < o.numdecoys; decoy++) {
-   source = &o.decoys[decoy];
+for(decoy=0; decoy < o.numdecoys; decoy++) {
+  source = &o.decoys[decoy];
 
-   /*do we even have to fill out this damn thing?  This is a raw packet, 
-     after all */
-   sock.sin_family = AF_INET;
-   sock.sin_port = htons(dport);
-   sock.sin_addr.s_addr = victim->s_addr;
+  memset((char *) packet, 0, sizeof(struct ip) + sizeof(udphdr_bsd));
 
+  udp->uh_sport = htons(sport);
+  udp->uh_dport = htons(dport);
+  udp->uh_ulen = htons(8 + datalen);
 
-   memset((char *) packet, 0, sizeof(struct ip) + sizeof(udphdr_bsd));
-
-   udp->uh_sport = htons(sport);
-   udp->uh_dport = htons(dport);
-   udp->uh_ulen = htons(8 + datalen);
-
-   /* Now the psuedo header for checksuming */
-   pseudo->source.s_addr = source->s_addr;
-   pseudo->dest.s_addr = victim->s_addr;
-   pseudo->proto = IPPROTO_UDP;
-   pseudo->length = htons(sizeof(udphdr_bsd) + datalen);
-
-   /* OK, now we should be able to compute a valid checksum */
-realcheck = in_cksum((unsigned short *)pseudo, 20 /* pseudo + UDP headers */ +
- datalen);
+  /* Now the pseudo header for checksuming */
+  pseudo->source.s_addr = source->s_addr;
+  pseudo->dest.s_addr = victim->s_addr;
+  pseudo->proto = IPPROTO_UDP;
+  pseudo->length = htons(sizeof(udphdr_bsd) + datalen);
+  
+  /* OK, now we should be able to compute a valid checksum */
+  realcheck = in_cksum((unsigned short *)pseudo, 20 /* pseudo + UDP headers */ +
+		       datalen);
 #if STUPID_SOLARIS_CHECKSUM_BUG
- udp->uh_sum = sizeof(struct udphdr_bsd) + datalen;
+  udp->uh_sum = sizeof(udphdr_bsd) + datalen;
 #else
-udp->uh_sum = realcheck;
+  udp->uh_sum = realcheck;
 #endif
 
-   /* Goodbye, pseudo header! */
-   memset(pseudo, 0, sizeof(*pseudo));
-
-   /* Now for the ip header */
-   ip->ip_v = 4;
-   ip->ip_hl = 5;
-   ip->ip_len = BSDFIX(sizeof(struct ip) + sizeof(udphdr_bsd) + datalen);
-   ip->ip_id = id;
-   ip->ip_ttl = myttl;
-   ip->ip_p = IPPROTO_UDP;
-   ip->ip_src.s_addr = source->s_addr;
-   ip->ip_dst.s_addr= victim->s_addr;
-
-   upi.ipck = in_cksum((unsigned short *)ip, sizeof(struct ip));
+  /* Goodbye, pseudo header! */
+  memset(pseudo, 0, sizeof(*pseudo));
+  
+  /* Now for the ip header */
+  ip->ip_v = 4;
+  ip->ip_hl = 5;
+  ip->ip_len = htons(sizeof(struct ip) + sizeof(udphdr_bsd) + datalen);
+  ip->ip_id = id;
+  ip->ip_ttl = myttl;
+  ip->ip_p = IPPROTO_UDP;
+  ip->ip_src.s_addr = source->s_addr;
+  ip->ip_dst.s_addr= victim->s_addr;
+  
+  upi.ipck = in_cksum((unsigned short *)ip, sizeof(struct ip));
 #if HAVE_IP_IP_SUM
-   ip->ip_sum = upi.ipck;
+  ip->ip_sum = upi.ipck;
 #endif
-
-   /* OK, now if this is the real she-bang (ie not a decoy) then
-      we stick all the inph0 in our upi */
-   if (decoy == o.decoyturn) {   
-     upi.iptl = 28 + datalen;
-     upi.ipid = id;
-     upi.sport = sport;
-     upi.dport = dport;
-     upi.udpck = realcheck;
-     upi.udplen = 8 + datalen;
-     upi.patternbyte = patternbyte;
-     upi.target.s_addr = ip->ip_dst.s_addr;
-   }
-   if (TCPIP_DEBUGGING > 1) {
-     log_write(LOG_STDOUT, "Raw UDP packet creation completed!  Here it is:\n");
-     readudppacket(packet,1);
-   }
-   if (TCPIP_DEBUGGING > 1)     
-     log_write(LOG_STDOUT, "\nTrying sendto(%d , packet, %d, 0 , %s , %d)\n",
-	    sd, BSDUFIX(ip->ip_len), inet_ntoa(*victim),
-	    (int) sizeof(struct sockaddr_in));
-
-   if ((res = sendto(sd, (const char *) packet, BSDUFIX(ip->ip_len), 0,
-		     (struct sockaddr *)&sock, (int) sizeof(struct sockaddr_in))) == -1)
-     {
-       perror("sendto in send_udp_raw_decoys");
-       return NULL;
-     }
-
-   if (TCPIP_DEBUGGING > 1) log_write(LOG_STDOUT, "successfully sent %d bytes of raw_tcp!\n", res);
- }
+  
+  /* OK, now if this is the real she-bang (ie not a decoy) then
+     we stick all the inph0 in our upi */
+  if (decoy == o.decoyturn) {   
+    upi.iptl = 28 + datalen;
+    upi.ipid = id;
+    upi.sport = sport;
+    upi.dport = dport;
+    upi.udpck = realcheck;
+    upi.udplen = 8 + datalen;
+    upi.patternbyte = patternbyte;
+    upi.target.s_addr = ip->ip_dst.s_addr;
+  }
+  if (TCPIP_DEBUGGING > 1) {
+    log_write(LOG_STDOUT, "Raw UDP packet creation completed!  Here it is:\n");
+    readudppacket(packet,1);
+  }
+  
+  if ((res = send_ip_packet(sd, eth, packet, ntohs(ip->ip_len))) == -1)
+    {
+      perror("send_ip_packet in send_closedupd_probe");
+      return NULL;
+    }
+}
 
 return &upi;
 
@@ -1856,11 +1892,7 @@ current_testno++;
 
 /* This next test doesn't work on Solaris because the lamers
    overwrite our ip_id */
-#if !defined(SOLARIS) && !defined(SUNOS) && !defined(IRIX)
-
-#ifdef WIN32
-if(!winip_corruption_possible()) {
-#endif
+#if !defined(SOLARIS) && !defined(SUNOS) && !defined(IRIX) && !defined(HPUX)
 
 /* Now lets see how they treated the ID we sent ... */
 AVs[current_testno].attribute = "RID";
@@ -1872,10 +1904,6 @@ else strcpy(AVs[current_testno].value, "F"); /* They fucked it up */
 
 current_testno++;
 
-#ifdef WIN32
-}
-#endif
-
 #endif
 
 /* Let us see if the IP checksum we got back computes */
@@ -1984,7 +2012,7 @@ int ipid_sequence(int numSamples, u16 *ipids, int islocalhost) {
 	if (ipid_diffs[i] % 256 == 0) /* Stupid MS */
 	  ipid_diffs[i] -= 256;
 	else
-	  ipid_diffs[i]--; /* Because on localhost the RST sent back ues an IPID */
+	  ipid_diffs[i]--; /* Because on localhost the RST sent back use an IPID */
       }
     }
   }
